==> qle-2.1.0.tar.gz: qle-2.1.0/src/parse.c <==
/*
    Quick Log Entry (qle)
    Copyright (C) 2025  Thomas Cort

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "adif.h"
#include "parse.h"
#include "re.h"
#include "str.h"
#include "transform.h"

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

struct qle_name_field qle_fields[QLE_NFIELDS] = {
	{ /* mode must be first in array */
		.adif_field = "mode",
		.qle_pattern = "^(CW|SSB|AM|FM)$",
		.transform = qle_transform_mode,
		.preserve = 1
	},
	{ /* rst_sent must be second in array */
		.adif_field = "rst_sent",
		.qle_pattern = "^[1-5][1-9][1-9]?$",
		.transform = qle_transform_rst,
		.preserve = 0 
	},
	{ /* rst_recv must be third in array */
		.adif_field = "rst_rcvd",
		.qle_pattern = NULL,
		.transform = qle_transform_rst,
		.preserve = 0 
	},
	{ /* band must be fourth in array */
		.adif_field = "band",
		.qle_pattern = NULL,
		.transform = NULL,
		.preserve = 1
	},
	{ /* sig must be fifth in array */
		.adif_field = "sig",
		.qle_pattern = NULL,
		.transform = NULL,
		.preserve = 0
	},
	{ /* sig must be sixth in array */
		.adif_field = "sig_info",
		.qle_pattern = NULL,
		.transform = NULL,
		.preserve = 0
	},
	{ /* sig must be seventh in array */
		.adif_field = "my_sig",
		.qle_pattern = NULL,
		.transform = NULL,
		.preserve = 1
	},
	{ /* sig must be eighth in array */
		.adif_field = "my_sig_info",
		.qle_pattern = NULL,
		.transform = NULL,
		.preserve = 1
	},
	{
		.adif_field = "pota_ref",
		.qle_pattern = "^POTA@.*$",
		.transform = qle_transform_sig,
		.preserve = 0
	},
	{
		.adif_field = "sota_ref",
		.qle_pattern = "^SOTA@.*$",
		.transform = qle_transform_sig,
		.preserve = 0
	},
	{
		.adif_field = "wwff_ref",
		.qle_pattern = "^WWFF@.*$",
		.transform = qle_transform_sig,
		.preserve = 0
	},
	{
		.adif_field = "gridsquare",
		.qle_pattern = "^grid@[A-R][A-R]([0-9][0-9]([A-X][A-X]([0-9][0-9])?)?)?$",
		.transform = qle_transform_at,
		.preserve = 0
	},
	{
		.adif_field = "my_gridsquare",
		.qle_pattern = "^mygrid@[A-R][A-R]([0-9][0-9]([A-X][A-X]([0-9][0-9])?)?)?$",
		.transform = qle_transform_at,
		.preserve = 1
	},
	{
		.adif_field = "cqz",
		.qle_pattern = "^cqz@[0-9]+$",
		.transform = qle_transform_at,
		.preserve = 0
	},
	{
		.adif_field = "my_cq_zone",
		.qle_pattern = "^mycqz@[0-9]+$",
		.transform = qle_transform_at,
		.preserve = 1
	},
	{
		.adif_field = "ituz",
		.qle_pattern = "^ituz@[0-9]+$",
		.transform = qle_transform_at,
		.preserve = 0
	},
	{
		.adif_field = "my_itu_zone",
		.qle_pattern = "^myituz@[0-9]+$",
		.transform = qle_transform_at,
		.preserve = 1
	},
	{
		.adif_field = "iota",
		.qle_pattern = "^IOTA@.*$",
		.transform = qle_transform_at,
		.preserve = 0
	},
	{
		.adif_field = "my_pota_ref",
		.qle_pattern = "^MYPOTA@.*$",
		.transform = qle_transform_mysig,
		.preserve = 1
	},
	{
		.adif_field = "my_sota_ref",
		.qle_pattern = "^MYSOTA@.*$",
		.transform = qle_transform_mysig,
		.preserve = 1
	},
	{
		.adif_field = "my_wwff_ref",
		.qle_pattern = "^MYWWFF@.*$",
		.transform = qle_transform_mysig,
		.preserve = 1
	},
	{
		.adif_field = "my_iota",
		.qle_pattern = "^MYIOTA@.*$",
		.transform = qle_transform_at,
		.preserve = 1 
	},
	{
		.adif_field = "notes",
		.qle_pattern = "\\{.*\\}",
		.transform = qle_transform_group,
		.preserve = 0
	},
	{
		.adif_field = "qslmsg",
		.qle_pattern = "\\[.*\\]",
		.transform = qle_transform_group,
		.preserve = 1
	},
	{
		.adif_field = "comment",
		.qle_pattern = "<.*>",
		.transform = qle_transform_group,
		.preserve = 0
	},
	{
		.adif_field = "freq",
		.qle_pattern = "^[0-9]+\\.[0-9]+$",
		.transform = qle_transform_freq,
		.preserve = 1
	},
	{
		.adif_field = "qso_date",
		.qle_pattern = "^[0-9]{8}$",
		.transform = qle_transform_all,
		.preserve = 1
	},
	{
		.adif_field = "time_on",
		.qle_pattern = "^(([0-9]{4})|([0-9]{6}))$",
		.transform = qle_transform_all,
		.preserve = 1
	},
	{
		.adif_field = "stx_string",
		.qle_pattern = "^,.*",
		.transform = qle_transform_no_first,
		.preserve = 1
	},
	{
		.adif_field = "srx_string",
		.qle_pattern = "^\\..*",
		.transform = qle_transform_no_first,
		.preserve = 0
	},
	{
		.adif_field = "name",
		.qle_pattern = "^!.*",
		.transform = qle_transform_no_first,
		.preserve = 0
	},
	{
		.adif_field = "skcc",
		.qle_pattern = "^#[0-9]+(C|T|S)?$",
		.transform = qle_transform_no_first,
		.preserve = 0
	},
	{
		.adif_field = "state",
		.qle_pattern = "^\\$.*",
		.transform = qle_transform_no_first,
		.preserve = 0
	},
	{
		.adif_field = "tx_pwr",
		.qle_pattern = "^[0-9]+(\\.[0-9]+)?W$",
		.transform = qle_transform_no_last,
		.preserve = 1
	},
	{
		.adif_field = "station_callsign",
		.qle_pattern = "^@[A-Z0-9/]+$",
		.transform = qle_transform_no_first,
		.preserve = 1
	},
	{
		.adif_field = "operator",
		.qle_pattern = "^%[A-Z0-9/]+$",
		.transform = qle_transform_no_first,
		.preserve = 1
	},
	{
		.adif_field = "contacted_op",
		.qle_pattern = "^\\^[A-Z0-9/]+$",
		.transform = qle_transform_no_first,
		.preserve = 0
	},
	{ /* call must always be the last item in this array */
		.adif_field = "call",
		.qle_pattern = "^[A-Z0-9/]+$",
		.transform = qle_transform_all,
		.preserve = 0
	}
};

void qle_rst_defaults(void) {

	char *rst_default;

	rst_default = strcmp(qle_fields[0].value, "CW") == 0 ? "599": "59";
	qle_strnlcpy(qle_fields[QLE_FIELD_INDEX_RST_RCVD].value, rst_default, QLE_TOKEN_SIZE, 0, strlen(rst_default));
	qle_strnlcpy(qle_fields[QLE_FIELD_INDEX_RST_SENT].value, rst_default, QLE_TOKEN_SIZE, 0, strlen(rst_default));
}

int qle_process_token(char *token) {

	int i;
	int rc;

	for (i = 0; i < QLE_NFIELDS; i++) {

		if (qle_fields[i].qle_pattern == NULL) {
			continue;
		}
		rc = qle_match(token, qle_fields[i].qle_pattern);
		if (rc == -1) { /* error */
			return -1;
		} else if (rc == 1) { /* match */
			qle_fields[i].transform(qle_fields[i].value, token, QLE_TOKEN_SIZE);
			return 0;
		}
	}

	return -1; /* no match */
}

int qle_process(FILE *input, FILE *output, FILE *error, int compact, int header, char *prelude, time_t input_mtime, char *input_file) {

	int ch;
	int i;
	int rc;
	int group;
	char *group_begin = "{[<";
	char *group_end = "}]>";
	static char token[QLE_TOKEN_SIZE];

	group = -1;
	memset(token, '\0', QLE_TOKEN_SIZE);
	for (i = 0; i < QLE_NFIELDS; i++) {
		memset(qle_fields[i].value, '\0', QLE_TOKEN_SIZE);
	}

	if (header == 1) {
		qle_adif_header(output, input_mtime, input_file);
	}

	if (prelude != NULL && prelude[0] != '\0') {
		rc = qle_process_token(prelude);
		if (rc == -1) {
			fprintf(error, "Syntax Error: '%s'\n", token);
			return -1;
		}
	}

	for (i = 0; (ch = fgetc(input)) != EOF && i < QLE_TOKEN_SIZE; i++) {

		if (group == -1 && qle_sindex(group_begin, ch) != -1) {
			group = qle_sindex(group_begin, ch);
		} else if (group != -1 && group == qle_sindex(group_end, ch)) {
			token[i] = ch;
			rc = qle_process_token(token);
			if (rc == -1) {
				fprintf(error, "Syntax Error: '%s'\n", token);
				return -1;
			}
			memset(token, '\0', QLE_TOKEN_SIZE);
			group = -1;
			i = -1;
			continue;
		} else if (group == -1 && isspace(ch)) {
			if (i > 0) {
				rc = qle_process_token(token);
				if (rc == -1) {
					fprintf(error, "Syntax Error: '%s'\n", token);
					return -1;
				}
				memset(token, '\0', QLE_TOKEN_SIZE);
			}
			if (ch == '\n') {
				qle_adif_record(output, compact);
			}
			i = -1;
			continue;
		}
		token[i] = ch;
	}

	if (i >= QLE_TOKEN_SIZE) {
		fprintf(error, "Input Too Large\n");
		return -1;
	} else if (ferror(input)) {
		fprintf(error, "Input Stream Error\n");
		return -1;
	} /* else EOF */

	qle_adif_record(output, compact);

	return 0;
}

==> qle-2.2.0.tar.gz: qle-2.2.0/src/parse.c <==
/*
    Quick Log Entry (qle)
    Copyright (C) 2025  Thomas Cort

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

    SPDX-License-Identifier: GPL-3.0-or-later
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "adif.h"
#include "parse.h"
#include "re.h"
#include "str.h"
#include "transform.h"

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

struct qle_name_field qle_fields[QLE_NFIELDS] = {
	{ /* mode must be first in array */
		.adif_field = "mode",
		.qle_pattern = "^(AM|ARDOP|ATV|CHIP|CLO|CONTESTI|CW|DIGITALVOICE|DOMINO|DYNAMIC|FAX|FM|FSK441|FT8|HELL|ISCAT|JT4|JT6M|JT9|JT44|JT65|MFSK|MSK144|MT63|OLIVIA|OPERA|PAC|PAX|PKT|PSK|PSK2K|Q15|QRA64|ROS|RTTY|RTTYM|SSB|SSTV|T10|THOR|THRB|TOR|V4|VOI|WINMOR|WSPR)$",
		.transform = qle_transform_mode,
		.preserve = 1
	},
	{ /* rst_sent must be second in array */
		.adif_field = "rst_sent",
		.qle_pattern = "^[1-5][1-9][1-9]?$",
		.transform = qle_transform_rst,
		.preserve = 0 
	},
	{ /* rst_recv must be third in array */
		.adif_field = "rst_rcvd",
		.qle_pattern = NULL,
		.transform = qle_transform_rst,
		.preserve = 0 
	},
	{ /* band must be fourth in array */
		.adif_field = "band",
		.qle_pattern = NULL,
		.transform = NULL,
		.preserve = 1
	},
	{ /* sig must be fifth in array */
		.adif_field = "sig",
		.qle_pattern = NULL,
		.transform = NULL,
		.preserve = 0
	},
	{ /* sig must be sixth in array */
		.adif_field = "sig_info",
		.qle_pattern = NULL,
		.transform = NULL,
		.preserve = 0
	},
	{ /* sig must be seventh in array */
		.adif_field = "my_sig",
		.qle_pattern = NULL,
		.transform = NULL,
		.preserve = 1
	},
	{ /* sig must be eighth in array */
		.adif_field = "my_sig_info",
		.qle_pattern = NULL,
		.transform = NULL,
		.preserve = 1
	},
	{ /* time_on must be ninth in array */
		.adif_field = "time_on",
		.qle_pattern = "^(([0-9]{4})|([0-9]{6}))(-(([0-9]{4})|([0-9]{6})))?$",
		.transform = qle_transform_time,
		.preserve = 1
	},
	{ /* time_off must be tenth in array */
		.adif_field = "time_off",
		.qle_pattern = NULL,
		.transform = NULL,
		.preserve = 0
	},
	{ /* qso_date must be eleventh in array */
		.adif_field = "qso_date",
		.qle_pattern = "^[0-9]{8}(-([0-9]{8}))?$",
		.transform = qle_transform_date,
		.preserve = 1
	},
	{ /* qso_date_off must be twelvth in array */
		.adif_field = "qso_date_off",
		.qle_pattern = NULL,
		.transform = NULL,
		.preserve = 0
	},
	{
		.adif_field = "pota_ref",
		.qle_pattern = "^POTA@.*$",
		.transform = qle_transform_sig,
		.preserve = 0
	},
	{
		.adif_field = "sota_ref",
		.qle_pattern = "^SOTA@.*$",
		.transform = qle_transform_sig,
		.preserve = 0
	},
	{
		.adif_field = "wwff_ref",
		.qle_pattern = "^WWFF@.*$",
		.transform = qle_transform_sig,
		.preserve = 0
	},
	{
		.adif_field = "gridsquare",
		.qle_pattern = "^grid@[A-R][A-R]([0-9][0-9]([A-X][A-X]([0-9][0-9])?)?)?$",
		.transform = qle_transform_at,
		.preserve = 0
	},
	{
		.adif_field = "my_gridsquare",
		.qle_pattern = "^mygrid@[A-R][A-R]([0-9][0-9]([A-X][A-X]([0-9][0-9])?)?)?$",
		.transform = qle_transform_at,
		.preserve = 1
	},
	{
		.adif_field = "cqz",
		.qle_pattern = "^cqz@[0-9]+$",
		.transform = qle_transform_at,
		.preserve = 0
	},
	{
		.adif_field = "my_cq_zone",
		.qle_pattern = "^mycqz@[0-9]+$",
		.transform = qle_transform_at,
		.preserve = 1
	},
	{
		.adif_field = "ituz",
		.qle_pattern = "^ituz@[0-9]+$",
		.transform = qle_transform_at,
		.preserve = 0
	},
	{
		.adif_field = "my_itu_zone",
		.qle_pattern = "^myituz@[0-9]+$",
		.transform = qle_transform_at,
		.preserve = 1
	},
	{
		.adif_field = "iota",
		.qle_pattern = "^IOTA@.*$",
		.transform = qle_transform_at,
		.preserve = 0
	},
	{
		.adif_field = "my_pota_ref",
		.qle_pattern = "^MYPOTA@.*$",
		.transform = qle_transform_mysig,
		.preserve = 1
	},
	{
		.adif_field = "my_sota_ref",
		.qle_pattern = "^MYSOTA@.*$",
		.transform = qle_transform_mysig,
		.preserve = 1
	},
	{
		.adif_field = "my_wwff_ref",
		.qle_pattern = "^MYWWFF@.*$",
		.transform = qle_transform_mysig,
		.preserve = 1
	},
	{
		.adif_field = "my_iota",
		.qle_pattern = "^MYIOTA@.*$",
		.transform = qle_transform_at,
		.preserve = 1 
	},
	{
		.adif_field = "notes",
		.qle_pattern = "\\{.*\\}",
		.transform = qle_transform_group,
		.preserve = 0
	},
	{
		.adif_field = "qslmsg",
		.qle_pattern = "\\[.*\\]",
		.transform = qle_transform_group,
		.preserve = 1
	},
	{
		.adif_field = "comment",
		.qle_pattern = "<.*>",
		.transform = qle_transform_group,
		.preserve = 0
	},
	{
		.adif_field = "freq",
		.qle_pattern = "^[0-9]+\\.[0-9]+$",
		.transform = qle_transform_freq,
		.preserve = 1
	},
	{
		.adif_field = "stx_string",
		.qle_pattern = "^,.*",
		.transform = qle_transform_no_first,
		.preserve = 1
	},
	{
		.adif_field = "srx_string",
		.qle_pattern = "^\\..*",
		.transform = qle_transform_no_first,
		.preserve = 0
	},
	{
		.adif_field = "name",
		.qle_pattern = "^!.*",
		.transform = qle_transform_no_first,
		.preserve = 0
	},
	{
		.adif_field = "skcc",
		.qle_pattern = "^#[0-9]+(C|T|S)?$",
		.transform = qle_transform_no_first,
		.preserve = 0
	},
	{
		.adif_field = "state",
		.qle_pattern = "^\\$.*",
		.transform = qle_transform_no_first,
		.preserve = 0
	},
	{
		.adif_field = "tx_pwr",
		.qle_pattern = "^[0-9]+(\\.[0-9]+)?W$",
		.transform = qle_transform_no_last,
		.preserve = 1
	},
	{
		.adif_field = "station_callsign",
		.qle_pattern = "^@[A-Z0-9/]+$",
		.transform = qle_transform_no_first,
		.preserve = 1
	},
	{
		.adif_field = "operator",
		.qle_pattern = "^%[A-Z0-9/]+$",
		.transform = qle_transform_no_first,
		.preserve = 1
	},
	{
		.adif_field = "contacted_op",
		.qle_pattern = "^\\^[A-Z0-9/]+$",
		.transform = qle_transform_no_first,
		.preserve = 0
	},
	{ /* call must always be the last item in this array */
		.adif_field = "call",
		.qle_pattern = "^[A-Z0-9/]+$",
		.transform = qle_transform_all,
		.preserve = 0
	}
};

void qle_rst_defaults(void) {

	char *rst_default;

	rst_default = strcmp(qle_fields[0].value, "CW") == 0 ? "599": "59";
	qle_strnlcpy(qle_fields[QLE_FIELD_INDEX_RST_RCVD].value, rst_default, QLE_TOKEN_SIZE, 0, strlen(rst_default));
	qle_strnlcpy(qle_fields[QLE_FIELD_INDEX_RST_SENT].value, rst_default, QLE_TOKEN_SIZE, 0, strlen(rst_default));
}

int qle_process_token(char *token) {

	int i;
	int rc;

	for (i = 0; i < QLE_NFIELDS; i++) {

		if (qle_fields[i].qle_pattern == NULL) {
			continue;
		}
		rc = qle_match(token, qle_fields[i].qle_pattern);
		if (rc == -1) { /* error */
			return -1;
		} else if (rc == 1) { /* match */
			qle_fields[i].transform(qle_fields[i].value, token, QLE_TOKEN_SIZE);
			return 0;
		}
	}

	return -1; /* no match */
}

int qle_process(FILE *input, FILE *output, FILE *error, int compact, int header, char *prelude, time_t input_mtime, char *input_file) {

	int ch;
	int i;
	int rc;
	int group;
	char *group_begin = "{[<";
	char *group_end = "}]>";
	static char token[QLE_TOKEN_SIZE];

	group = -1;
	memset(token, '\0', QLE_TOKEN_SIZE);
	for (i = 0; i < QLE_NFIELDS; i++) {
		memset(qle_fields[i].value, '\0', QLE_TOKEN_SIZE);
	}

	if (header == 1) {
		qle_adif_header(output, input_mtime, input_file);
	}

	if (prelude != NULL && prelude[0] != '\0') {
		rc = qle_process_token(prelude);
		if (rc == -1) {
			fprintf(error, "Syntax Error: '%s'\n", token);
			return -1;
		}
	}

	for (i = 0; (ch = fgetc(input)) != EOF && i < QLE_TOKEN_SIZE; i++) {

		if (group == -1 && qle_sindex(group_begin, ch) != -1) {
			group = qle_sindex(group_begin, ch);
		} else if (group != -1 && group == qle_sindex(group_end, ch)) {
			token[i] = ch;
			rc = qle_process_token(token);
			if (rc == -1) {
				fprintf(error, "Syntax Error: '%s'\n", token);
				return -1;
			}
			memset(token, '\0', QLE_TOKEN_SIZE);
			group = -1;
			i = -1;
			continue;
		} else if (group == -1 && isspace(ch)) {
			if (i > 0) {
				rc = qle_process_token(token);
				if (rc == -1) {
					fprintf(error, "Syntax Error: '%s'\n", token);
					return -1;
				}
				memset(token, '\0', QLE_TOKEN_SIZE);
			}
			if (ch == '\n') {
				qle_adif_record(output, compact);
			}
			i = -1;
			continue;
		}
		token[i] = ch;
	}

	if (i >= QLE_TOKEN_SIZE) {
		fprintf(error, "Input Too Large\n");
		return -1;
	} else if (ferror(input)) {
		fprintf(error, "Input Stream Error\n");
		return -1;
	} /* else EOF */

	qle_adif_record(output, compact);

	return 0;
}
